% Generate 2D continuous path through 2D data depending on type that does
% not repeat any node in the path
% dimension : dimensions of 2D array
% path : list of x,y points
% discont : 1D array (same size as path), says if path is discontinuous at
% that point

% type = 1 : generate continuous path N-S-N
% type = 2 : generate continuous path W-E-W
% type = 3 : generate discontinuous path N-S
% type = 4 : generate discontinuous path S-N
% type = 5 : generate discontinuous path W-E
% type = 6 : generate discontinuous path E-W
% type = 11 : generate path clockwise spiral
% type = 12 : generate path anticlockwise spiral

function [path,discont] = generate_2D_path(dimension, type)

s = size(dimension);
if(s ~= [1 2])
    disp('Input dimension array must be 2 elements in 1 row');
    path = 0;
    discont = 1;
    return;
end

n1 = dimension(1,1);
n2 = dimension(1,2);

path = zeros(n1*n2,2);
discont = zeros(n1*n2,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 1) % generate continuous path N-S-N
    count = 0;
    flip = 1;
    for i=1:n2
        if(flip == 1)
            path(count+1:count+n1,1) = 1:n1;
        else
            path(count+1:count+n1,1) = n1:-1:1;
        end
        
        path(count+1:count+n1,2) = i;
        
        count = count + n1;
        flip = (-1)*flip;
    end
    discont(1,1) = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 2) % generate continuous path W-E-W
    count = 0;
    flip = 1;
    for i=1:n1
        if(flip == 1)
            path(count+1:count+n2,2) = 1:n2;
        else
            path(count+1:count+n2,2) = n2:-1:1;
        end
        
        path(count+1:count+n2,1) = i;
        
        count = count + n2;
        flip = (-1)*flip;
    end
    discont(1,1) = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 3) % generate discontinuous path N-S
    count = 0;
    for i=1:n2
        discont(count+1,1) = 1;
        path(count+1:count+n1,1) = 1:n1;
        path(count+1:count+n1,2) = i;
        
        count = count + n1;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 4) % generate discontinuous path S-N
    count = 0;
    for i=1:n2
        discont(count+1,1) = 1;
        path(count+1:count+n1,1) = n1:-1:1;
        path(count+1:count+n1,2) = i;
        
        count = count + n1;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 5) % generate discontinuous path W-E
    count = 0;
    for i=1:n1
        discont(count+1,1) = 1;
        path(count+1:count+n2,2) = 1:n2;  
        path(count+1:count+n2,1) = i;
        
        count = count + n2;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 6) % generate discontinuous path E-W
    count = 0;
    for i=1:n1
        discont(count+1,1) = 1;
        path(count+1:count+n2,2) = n2:-1:1;  
        path(count+1:count+n2,1) = i;
        
        count = count + n2;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 11) % generate path clockwise spiral
    count = 0;
    vertex1 = [1 1];
    vertex2 = [n1 n2];
    
    while((vertex1(1,1)<=vertex2(1,1)) && (vertex1(1,2)<=vertex2(1,2)))
        
        % Top row
        dn2 = vertex2(1,2) - vertex1(1,2) + 1;
        path(count+1:count+dn2,2) = vertex1(1,2):vertex2(1,2);
        path(count+1:count+dn2,1) = vertex1(1,1);
        
        count = count + dn2;
        vertex1(1,1) = vertex1(1,1) + 1;
        
        % Right column
        dn1 = vertex2(1,1) - vertex1(1,1) + 1;
        if(dn1 >= 1)
            path(count+1:count+dn1,1) = vertex1(1,1):vertex2(1,1);
            path(count+1:count+dn1,2) = vertex2(1,2);
            
            count = count + dn1;
            vertex2(1,2) = vertex2(1,2) - 1;
        else
            break;
        end
        
        % Bottom row
        dn2 = vertex2(1,2) - vertex1(1,2) + 1;
        if(dn2>=1)
            path(count+1:count+dn2,2) = vertex2(1,2):-1:vertex1(1,2);
            path(count+1:count+dn2,1) = vertex2(1,1);
            
            count = count + dn2;
            vertex2(1,1) = vertex2(1,1) - 1;
        else
            break;
        end
        
        % Left column
        dn1 = vertex2(1,1) - vertex1(1,1) + 1;
        if(dn1 >= 1)
            path(count+1:count+dn1,1) = vertex2(1,1):-1:vertex1(1,1);
            path(count+1:count+dn1,2) = vertex1(1,2);
            
            count = count + dn1;
            vertex1(1,2) = vertex1(1,2) + 1;
        else
            break;
        end 
    end
    discont(1,1) = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(type == 12) % generate path anticlockwise spiral
    count = 0;
    vertex1 = [1 1];
    vertex2 = [n1 n2];
    
    while((vertex1(1,1)<=vertex2(1,1)) && (vertex1(1,2)<=vertex2(1,2)))
        
        % Left column
        dn1 = vertex2(1,1) - vertex1(1,1) + 1;
        path(count+1:count+dn1,1) = vertex1(1,1):vertex2(1,1);
        path(count+1:count+dn1,2) = vertex1(1,2);

        count = count + dn1;
        vertex1(1,2) = vertex1(1,2) + 1;
        
        % Bottom row
        dn2 = vertex2(1,2) - vertex1(1,2) + 1;
        if(dn2>=1)
            path(count+1:count+dn2,2) = vertex1(1,2):vertex2(1,2);
            path(count+1:count+dn2,1) = vertex2(1,1);
            
            count = count + dn2;
            vertex2(1,1) = vertex2(1,1) - 1;
        else
            break;
        end
        
        % Right column
        dn1 = vertex2(1,1) - vertex1(1,1) + 1;
        if(dn1 >= 1)
            path(count+1:count+dn1,1) = vertex2(1,1):-1:vertex1(1,1);
            path(count+1:count+dn1,2) = vertex2(1,2);
            
            count = count + dn1;
            vertex2(1,2) = vertex2(1,2) - 1;
        else
            break;
        end
        
        % Top row
        dn2 = vertex2(1,2) - vertex1(1,2) + 1;
        if(dn2 >= 1)
            path(count+1:count+dn2,2) = vertex2(1,2):-1:vertex1(1,2);
            path(count+1:count+dn2,1) = vertex1(1,1);

            count = count + dn2;
            vertex1(1,1) = vertex1(1,1) + 1;
        else
            break;
        end
        
    end
    discont(1,1) = 1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

return;

end