% This function computes the filter coefficients for non-stationary AR
% model, applies them on the input data, & returns the reconstructed data & the
% residual. It works on 1D datasets.

function [output,res,status] = nonstationary_AR_1D(input,box_len,order)

N = size(input);
if(N(1,2) ~= 1)
    print('Function "nonstationary_AR_1D" accepts 1D input only. Exiting...');
    status = 1;
    return;
end
N = N(1,1);
p = order;
b = box_len;

% Initialize outputs
output = zeros(N,1);

UT = zeros(b,p);
tol2 = 1e-5;

phi = zeros(p,1);
u = zeros(b,1);
for i = 1:(b+p-1)
    % Construct u
    disp(i);
    for j = 1:b
        if(i-b+j >= 1)
            u(j,1) = input(i-b+j,1);
        else
            u(j,1) = input(1,1);
        end
    end
    % Construct UT
    for j = 1:p
        for k = 1:b
            if(i-j-b+k >= 1)
                UT(k,j) = input(i-j-b+k,1);
            else
                UT(k,j) = input(1,1);
            end
        end
    end
    
    % Steepest descent iterations
    [phi,r] = steepest_descent(UT,u,phi,tol2);
    output(i,1) = u(b,1) + r(b,1);
end
for i = (b+p):N
    % Construct u
    disp(i);
    u = input(i-b+1:i,1);
    % Construct UT
    for j = 1:p
        UT(:,j) = input(i-j-b+1:i-j,1);
    end
    
    % Steepest descent iterations
    [phi,r] = steepest_descent(UT,u,phi,tol2);
    output(i,1) = u(b,1) + r(b,1);
end
res = input - output;
status = 0;
return;

end