% This function computes the filter coefficients for non-stationary SAR
% model, applies them on the input data, & returns the reconstructed data & the
% residual. It works on 1D datasets.

function [output,res,status] = nonstationary_SAR_1D(input,order,gamma)

N = size(input);
if(N(1,2) ~= 1)
    print('Function "nonstationary_WAR_1D" accepts 1D input only. Exiting...');
    status = 1;
    return;
end
N = N(1,1);
p = order;

% Initialize outputs
output = zeros(N,1);

% Handle first point
phi = zeros(p,1);
phi(1,1) = 1;
output(1,1) = input(1,1);

% Start streaming
for i = 2:N
    % Get u
    u = get_d_tilda_1D(input,i,order);
    
    % Update phi
    phi = phi +((input(i,1) - u'*phi)/(u'*u + gamma^2))*u;
    
    % Update output at current data location
    output(i,1) = u'*phi;
end

res = input - output;
status = 0;
return;

end

function d_tilda = get_d_tilda_1D(A,index,order)

d_tilda = zeros(order,1);
if((index-order) >= 1)
    d_tilda = A((index-order):(index-1),1);
else
    d_tilda(1:(order-index+1),1) = A(1,1);
    d_tilda((order-index+2):order,1) = A(1:(index-1),1);
end
return;
end