clear all
close all
clc

A = load('Dat/wgstack.txt');
A = A(250,:);
A = A';
n_A = size(A);
n_A = n_A(1,1);
A = A/sqrt((1/n_A)*A'*A);

%Interp A with cubic spline
f = 1;
x = [0:f:f*n_A]';
y = x;
B = zeros(f*n_A+1,1);
for i=1:n_A
    y(i+1,1) = A(i,1);
end

B = spline(x,y,0:f*n_A);
B = B';
x = [0:f*n_A]';

figure('units','normalized','outerposition',[0 0 1 1])
plot(x,B,'-red','linewidth',1.5)
grid on;
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
title('Time Slice (t = 1 s) of a 2D Seismic Stack','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')

print -depsc2 Fig/single-trace
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stationary AR
order = 5;
[C,PHI,RES,status] = stationary_AR_1D(B,order);
RES = 1.5*RES;

figure('units','normalized','outerposition',[0 0 1 1])
subplot(2,1,1)
plot(x,B,'-red','linewidth',1.5)
hold on;
plot(x,C,'-black','linewidth',1)
grid on
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
title('Time Slice (t = 1 s) of a 2D Seismic Stack','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('Input data','Reconstructed data');
set(h_legend,'FontSize',12);

subplot(2,1,2)
plot(x,RES,'-blue','linewidth',1.5)
grid on;
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('1.5 x Prediction error');
set(h_legend,'FontSize',12);

print -depsc2 Fig/single-trace-SPE
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Non-Stationary AR
order = 5;
[C,RES,status] = nonstationary_AR_1D(B,10,order);
RES = 1.5*RES;

figure('units','normalized','outerposition',[0 0 1 1])
subplot(2,1,1)
plot(x,B,'-red','linewidth',1.5)
hold on;
plot(x,C,'-black','linewidth',1)
grid on
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
title('Time Slice (t = 1 s) of a 2D Seismic Stack','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('Input data','Reconstructed data');
set(h_legend,'FontSize',12);

subplot(2,1,2)
plot(x,RES,'-blue','linewidth',1.5)
grid on;
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('1.5 x Prediction error');
set(h_legend,'FontSize',12);

print -depsc2 Fig/single-trace-NSPE
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Weighted Non-Stationary AR

order = 5;
[C,RES,status] = nonstationary_WAR_1D(B,10,order);
RES = 1.5*RES;

figure('units','normalized','outerposition',[0 0 1 1])
subplot(2,1,1)
plot(x,B,'-red','linewidth',1.5)
hold on;
plot(x,C,'-black','linewidth',1)
grid on
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
title('Time Slice (t = 1 s) of a 2D Seismic Stack','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('Input data','Reconstructed data');
set(h_legend,'FontSize',12);

subplot(2,1,2)
plot(x,RES,'-blue','linewidth',1.5)
grid on;
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('1.5 x Prediction error');
set(h_legend,'FontSize',12);

print -depsc2 Fig/single-trace-WNSPE
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Streaming Non-Stationary AR

order = 5;
gamma = 0.8;
[C,RES,status] = nonstationary_SAR_1D(B,order,gamma);
RES = 1.5*RES;

figure('units','normalized','outerposition',[0 0 1 1])
subplot(2,1,1)
plot(x,B,'-red','linewidth',1.5)
hold on;
plot(x,C,'-black','linewidth',1)
grid on
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
title('Time Slice (t = 1 s) of a 2D Seismic Stack','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('Input data','Reconstructed data');
set(h_legend,'FontSize',12);

subplot(2,1,2)
plot(x,RES,'-blue','linewidth',1.5)
grid on;
axis([x(1) x(f*n_A+1) -max(abs(B))*2 max(abs(B))*2])
xlabel('Kilometers','FontSize',14,'FontWeight','normal')
ylabel('Amplitude','FontSize',14,'FontWeight','normal')
ax = gca();
set(ax,'XTick',1:100:f*n_A)
xticklabel = [1:100:f*n_A]*0.0335/f;
set(ax,'XTickLabel',sprintf('%1.2f|',xticklabel),'FontSize',12,'FontWeight','normal')
h_legend = legend('1.5 x Prediction error');
set(h_legend,'FontSize',12);

print -depsc2 Fig/single-trace-SNSPE
close all
