function [output,res,status] = streaming_AR_2D_colinterp(A,col2interp,colref,order,box,gamma,beta)

s1 = size(col2interp);
res = A*0;
weight = get_weight(beta,order);

for i=1:s1(1,2)
    col = col2interp(1,i);
    disp(col);
    if(colref == 0)
        colref1 = col - 1;
    else
        colref1 = colref;
    end
    [A,res] = interpcol_decay(A,colref1,col,order,box,gamma,weight);
end

output = A;
status = 0;

return;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to interpolate 1 column
function [output,res] = interpcol_decay(A,col0,col,order,box,gamma,weight)

s = size(A);
output = A;
res = A*0;

n_coeff = (order(1,2)-1)*order(1,1);
s2 = size(box);

dimension = [s(1,1) col0];
path = generate_2D_path(dimension, 5);

count = 1;
for i=1:s(1,1)
    
    % Get new data element & corresponding row & col
    d_new_row = path(count,1);
    d_new_col = path(count,2);
    d_new = A(d_new_row,d_new_col);
    
    % Get corners of bounding box
    row_start = d_new_row - ceil(s2(1,1)/2) + 1;
    col_start =  d_new_col;
    row_end = d_new_row + floor(s2(1,1)/2);
    col_end =  d_new_col + s2(1,2) - 1;

    % Adjust corners of bounding box
    if(row_start < 1)
        row_start = 1;
    end
    if(row_end > s(1,1))
        row_end = s(1,1);
    end
    if(col_start < 1)
        col_start = 1;
    end
    if(col_end > s(1,2))
        col_end = s(1,2);
    end
    % Setup regression matrix & data vector
    n_cols = n_coeff;
    n_rows = (row_end - row_start + 1)*(col_end - col_start + 1);
    DT = zeros(n_rows,n_cols);
    d = zeros(n_rows,1);

    count1 = 1;
    for j = col_start:col_end
        for k = row_start:row_end
            DT(count1,:) = get_d_tilda_colinterp(A,k,j,order(1,1),order(1,2));
            d(count1,:) = A(k,j);
            count1 = count1 + 1;
        end
    end

    % Solve for phi using steepest descent
    phi = zeros(n_coeff,1) + 1/n_coeff;
    phi = steepest_descent(DT,d,phi,1e-10);
    
    % Update output at current data location
    d_tilda = get_d_tilda_colinterp(A,d_new_row,d_new_col,order(1,1),order(1,2));
    res(d_new_row,d_new_col) = d_new - d_tilda'*phi;
    
    count = count + 1;
    
    for j=2:col0
        % Get new data element & corresponding row & col
        d_new_row = path(count,1);
        d_new_col = path(count,2);
        d_new = A(d_new_row,d_new_col);

        % Get d_tilda
        d_tilda = get_d_tilda_colinterp(A,d_new_row,d_new_col,order(1,1),order(1,2));

        % Update phi
        phi = phi +((d_new - d_tilda'*phi)/(d_tilda'*d_tilda + gamma^2))*d_tilda;
        
        % Update output at current data location
        res(d_new_row,d_new_col) = d_new - d_tilda'*phi;
        
        count = count + 1;
    end
    
    % Update missing data location
    d_tilda = get_d_tilda_colinterp(A,i,col,order(1,1),order(1,2));
    phi_weight = phi.*(weight.^(col-col0));
    output(i,col) = d_tilda'*phi_weight;
    
end

return;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to evaluate weights for filter decay
function weight = get_weight(beta,order)

n_coeff = (order(1,2)-1)*order(1,1);
weight = zeros(n_coeff,1);

count = 1;
i_ref = ceil(order(1,1)/2);
j_ref = order(1,2);
for j=1:(order(1,2)-1)
    dj = j - j_ref;
    for i=1:order(1,1)
        di = i - i_ref;
        weight(count,1) = beta^(abs(di) + abs(dj));
        count = count + 1;
    end
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to get d_tilda for col interpolation
function d_tilda = get_d_tilda_colinterp(A,row,col,n1,n2)

s = size(A);
n_coeff = (n2-1)*n1;
d_tilda = zeros(n_coeff,1);

% Update box corners to fetch old data vector
row_start = row - ceil(n1/2) + 1;
col_start = col - n2 + 1;
row_end = row + floor(n1/2);
col_end = col;

% Update old data vector
count = 0;
for i2=col_start:(col_end-1)
    % Update index2 with proper handling of boundary
    index2 = i2;
    if(i2<1)
        index2 = 1;
    end

    for i1=row_start:row_end
        % Update index1 with proper handling of boundary
        index1 = i1;
        if(i1<1)
            index1 = 1;
        end
        if(i1>s(1,1))
            index1 = s(1,1);
        end

        count = count + 1;
        d_tilda(count,1) = A(index1,index2);
    end
end
return;
end