% This function computes the filter coefficients for non-stationary AR
% model in streaming manner with the fast rank one update formula, applies 
% them on the A data, & returns the reconstructed data & the
% residual. It works on 2D datasets. A path is also A to use for
% the rank 1 update formula

function [output,res,status] = streaming_AR_2D_path(A,order,box,path,discont,gamma)

s = size(A);
output = zeros(s(1,1),s(1,2));

n_coeff = (order(1,2)-1)*order(1,1) + ceil(order(1,1)/2) - 1;
phi = zeros(n_coeff,1);
%gamma = .02; % To be changed

s1 = size(path);
s2 = size(box);
for i=1:s1(1,1)
    if(discont(i,1)==1)
        % Get new data element & corresponding row & col
        d_new_row = path(i,1);
        d_new_col = path(i,2);
        
        % Get corners of bounding box
        row_start = d_new_row - ceil(s2(1,1)/2) + 1;
        col_start =  d_new_col - ceil(s2(1,2)/2) + 1;
        row_end = d_new_row + floor(s2(1,1)/2);
        col_end =  d_new_col + floor(s2(1,2)/2);
        
        % Adjust corners of bounding box
        if(row_start < 1)
            row_start = 1;
        end
        if(row_end > s(1,1))
            row_end = s(1,1);
        end
        if(col_start < 1)
            col_start = 1;
        end
        if(col_end > s(1,2))
            col_end = s(1,2);
        end
        
        % Setup regression matrix & data vector
        n_cols = n_coeff;
        n_rows = (row_end - row_start + 1)*(col_end - col_start + 1);
        DT = zeros(n_rows,n_cols);
        d = zeros(n_rows,1);
        
        count = 1;
        for j = col_start:col_end
            for k = row_start:row_end
                DT(count,:) = get_d_tilda(A,k,j,order(1,1),order(1,2));
                d(count,:) = A(k,j);
                count = count + 1;
            end
        end
        
        % Solve for phi using steepest descent
        phi = zeros(n_coeff,1) + 1/n_coeff;
        phi = steepest_descent(DT,d,phi,1e-10);
        
        % Get d_tilda for current location
        d_tilda = get_d_tilda(A,d_new_row,d_new_col,order(1,1),order(1,2));
        
        % Update output at current data location
        output(d_new_row,d_new_col) = d_tilda'*phi;
        
    else
        % Get new data element & corresponding row & col
        d_new_row = path(i,1);
        d_new_col = path(i,2);
        d_new = A(d_new_row,d_new_col);
        
        % Get d_tilda
        d_tilda = get_d_tilda(A,d_new_row,d_new_col,order(1,1),order(1,2));
        
        % Update phi
        phi = phi +((d_new - d_tilda'*phi)/(d_tilda'*d_tilda + gamma^2))*d_tilda;
        
        % Update output at current data location
        output(d_new_row,d_new_col) = d_tilda'*phi;
    end
end

res = A - output;
status = 0;
return;

end
